#include <userint.h>
#include <ansi_c.h>
#include "prthires.h"

#define     MAXLEN          256      /* max. length of any string */
#define     CONV_CONST      254      /* conversion constant to convert
                                        from millimeters to inches */
#define     TRUE            1
#define     FALSE           0

#define     kEndOfList      -1
#define     kErrorNo        0

/* maximum and min. point sizes */

#define     MAX_POINT_SIZE  255
#define     MIN_POINT_SIZE  1

/**** Attributes of the duplicated panel ****/

#define     kDupPanelTitle  ""
#define     kDupPanelTop    300
#define     kDupPanelLeft   300
#define     kDupPanelHeight 300
#define     kDupPanelWidth  300

#define     Min(a,b)        ((a) > (b) ? (b) : (a))
#define     Max(a,b)        ((a) > (b) ? (a) : (b))

/* Attributes of the duplicate control */

#define     kDupCtrlTop     50
#define     kDupCtrlLeft    50

typedef enum {
        kHeight,
        kWidth
} Dimension;

typedef enum {
        kIncrease,
        kDecrease
} Direction;

/* A table of font attributes */

int fontNameAttributes [] = {ATTR_LABEL_FONT,
                             ATTR_XYLABEL_FONT,
                             ATTR_XYNAME_FONT, kEndOfList};

/* Error checking macro */

#define     errChk(functionCall)    {error = (functionCall); if (error<0) {goto Error;}}

/***************************************************************************/

extern int CVIFUNC_C GetValidatedPrintAttribute (int attribute, int *value);
    /*  undocumented User Interface Library function: analogous to
        GetPrintAttribute(), except that the attributes are bound to the
        current printer, and validated -- should return the same values as in
        the "Actual:" section of the print dialog which pops up when the last
        parameter to PrintPanel() or PrintCtrl() is non-zero.
    */
extern int CVIFUNC_C ScaleTextPlots (int panel, int control, double factor);
    /*  undocumented User Interface Library function: scales the font's point
        size of all of a graph's text plots by a scaling factor, which must be
        greater than zero. It's used to preserve the same apparent size of
        text plots, in the printed graph.
    */

/*********************************/
/*  PRIVATE FUNCTION PROTOTYPES  */
/*********************************/

static int  ConvertMilliMetersToDots (int dataInMilliMeters, Dimension dim);
static int  ComputeScalingFactorsFromPrintAttributes (int origRectWidth, int origRectHeight,
                                                      float *widthScalingFactor, float *heightScalingFactor);
static int  GetScalingFactorsFromBoundingRect (int panel, int controlId,
                                               float *widthScalingFactor, float *heightScalingFactor);
static int  DuplicateAndSetTheSizeOfTheControl (int origPanel, int origCtrl,
                                                int *dupPanel, int *dupCtrl,
                                                float widthScalingFactor, float heightScalingFactor);
static void MakeNumTableEntriesEven (char **table);
static int  SubstituteFonts (int panel, int controlId, char **fontSubstitutionTable);
static int  SearchAndSubstituteFont (int panel, int controlId, char *fontToSearch,
                                                               char *fontToSubstitute);
static int  CaseInSensitiveStringCompare (char *string1, char *string2);
static int  HandleTextPlots (int panel, int controlId, float widthScalingFactor,
                                                       float heightScalingFactor);
static int  GetNewPointSizeAfterChangingDimensions (int panel, int ctrl, int newDimension,
                                                               float scalingFactor, int whichDimension);
static int  UpdateTitleLabel (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl,
                                                                         float widthScalingFactor,
                                                                         float heightScalingFactor);
static int  UpdateAxisLabelsAndNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl,
                                                                   int   textCtrl,
                                                                   float widthScalingFactor,
                                                                   float heightScalingFactor);
static int  HandleXAndYAxisNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl, char *xAxisName, char *yAxisName,
                                  float whichFactor, float widthScalingFactor, float heightScalingFactor,
                                  int *newDimensionToSetFontSize);
static int  HandleXOrYAxisNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl, char *axisName, float whichFactor,
                                 float widthScalingFactor, float heightScalingFactor,
                                 int *newDimensionToSetFontSize);
static int  UpdateGraphOrStripChartLabelsToNewPointSizes (int origPanel, int origCtrl, int dupPanel, int dupCtrl,
                                                                         float widthScalingFactor,
                                                                         float heightScalingFactor);
static int  PlaceLabel (int panel, int controlId, int attrCtrlDim, int attrCtrl, int attrLabel, float percentage);
static int  ComputeLabelPlacementPercentage (int panel, int controlId, int attrCtrlDim,
                                             int attrCtrl, int attrLabel, float *percentage);
static int  PlaceControlLabelAtTheRightPlace (int origPanel, int origCtrl, int dupPanel, int dupCtrl);

/***************************************************************************/

static int ComputeLabelPlacementPercentage (int panel, int controlId, int attrCtrlDim,
                                            int attrCtrl, int attrLabel, float *percentage)

{
    int     ctrlDimSize, labelOffset, ctrlOffset;
    float   placementDifference;
    int     error = kErrorNo;

    errChk (GetCtrlAttribute (panel, controlId, attrCtrlDim, &ctrlDimSize));
    errChk (GetCtrlAttribute (panel, controlId, attrCtrl, &ctrlOffset));
    errChk (GetCtrlAttribute (panel, controlId, attrLabel, &labelOffset));

    placementDifference = (float)(labelOffset - ctrlOffset);
    *percentage = ((float)placementDifference/(float)ctrlDimSize)*(float)100;

Error:
    return error;
}

/***************************************************************************/

static int GetNewPointSizeAfterChangingDimensions (int panel, int ctrl, int newDimension,
                                                   float scalingFactor, int whichDimension)

/* Once a control has been enlarged or reduced, one of the things that have to be
   modified are the fonts.  This algorithm returns a modified point size depending
   upon whether the control has been enlarged or reduced

   It accepts as arguments...

   scalingFactor  - the scaling factor by which the control has been modified.  This
                    will serve as a heuristic to start from a paricular font size and
                    go up or down from there.
   newDimension   - the dimension (width or height) of the new control that we want to
                    finally reach.
   whichDimension - Is it the height or the width by which we grow or decrease the font? */

{
    int         pointSize;
    int         currentDimension;                   /* Dimension which varies according to the point size */
    int         crossed = FALSE;
    int         maxDimension;                       /* max. dimension seen so far */
    int         maxDimensionPointSize;              /* point size corresponding to the max. dimension */
    Direction   dir;
    int         error = kErrorNo;

    /* Start off from a point size and try to reach the desired dimension.  There are 2 cases :-

    Case 1 : You are at a higher dimension and want to go down to a lower one.  Begin by starting
             to decrease the point size in decrements of 1.  The moment you cross over the dimension,
             pick the point size at which you crossed over.

    Case 2 : Trying to reach a higher dimension from a lower one.  Keep increasing the point size by
             increments of 1.  At the same time keep track of the max. dimension seen so far and
             the point size corresponding to that dimension.  Moment you cross over, the point size
             corresponding to that maximum dimension is the desired point size */

    errChk (GetCtrlAttribute (panel, ctrl, ATTR_TEXT_POINT_SIZE, &pointSize));

    if (scalingFactor)
        pointSize = (int)((float)pointSize * scalingFactor);

    if (pointSize > MAX_POINT_SIZE)
        pointSize = MAX_POINT_SIZE;
    if (pointSize < MIN_POINT_SIZE)
        pointSize = MIN_POINT_SIZE;

    errChk (SetCtrlAttribute (panel, ctrl, ATTR_TEXT_POINT_SIZE, pointSize));
    errChk (GetCtrlAttribute (panel, ctrl, whichDimension, &currentDimension));
    currentDimension -= 2;

    if (currentDimension == newDimension)
        return pointSize;

    if (currentDimension < newDimension)
        dir = kIncrease;
    else
        dir = kDecrease;

    crossed = ((newDimension == currentDimension) || ((dir == kIncrease) ? (currentDimension>newDimension)
                                                                   : (currentDimension<newDimension)));

    maxDimension = 0;

    while (!crossed)
    {
        if (dir == kIncrease)
        {
            if (currentDimension >= maxDimension)
            {
                maxDimension = currentDimension;
                maxDimensionPointSize = pointSize;
            }
        }
        pointSize = (newDimension > currentDimension) ? pointSize+1 : pointSize-1;

        if (pointSize > MAX_POINT_SIZE)
            return maxDimensionPointSize;

        if (pointSize < MIN_POINT_SIZE)
            return MIN_POINT_SIZE;

        errChk (SetCtrlAttribute (panel, ctrl, ATTR_TEXT_POINT_SIZE, pointSize));
        errChk (GetCtrlAttribute (panel, ctrl, whichDimension, &currentDimension));
        currentDimension -= 2;
        crossed = ((newDimension == currentDimension) || ((dir == kIncrease) ? (currentDimension>newDimension)
                                                                   : (currentDimension<newDimension)));
    }

    /* Just return the previous font size the moment you crossed over the limit */

    if (newDimension == currentDimension)
        maxDimension = currentDimension;

    return (((dir == kIncrease) && (newDimension != currentDimension)) ? maxDimensionPointSize : pointSize);

Error:
    return error;
}

/***************************************************************************/

static int HandleTextPlots (int dupPanel, int dupCtrl, float widthScalingFactor,
                                                 float heightScalingFactor)

{   int     ctrlStyle;
    int     error = kErrorNo;

    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_CTRL_STYLE, &ctrlStyle));

    if (ctrlStyle == CTRL_GRAPH)
        errChk (ScaleTextPlots (dupPanel, dupCtrl, Min (widthScalingFactor, heightScalingFactor)));

Error:
    return error;
}

/***************************************************************************/

static int UpdateTitleLabel (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl,
                                                                        float widthScalingFactor,
                                                                        float heightScalingFactor)

/* Updates the font size of the title of the control */

{
    char    title [MAXLEN] = {'\0'};
    char    titleFontName [MAXLEN] = {'\0'};
    int     origLabelWidth, origLabelHeight, newLabelWidth, newLabelHeight;
    float   whichFactor;
    int     newDimensionToSetFontSize, newPointSize, titlePointSize;
    int     textMsgDimension;
    int     sizeToText;
    int     error = kErrorNo;

    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_TEXT, title));

    if (strlen (title))
    {
        /* copy label text to text ctrl */

        errChk (SetCtrlVal (dupPanel, textCtrl, title));

        errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_FONT, titleFontName));
        errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_FONT, titleFontName));
        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_POINT_SIZE, &titlePointSize));
        errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_POINT_SIZE, titlePointSize));

        /* calculate the minimum scaling factor */

        whichFactor = Min (widthScalingFactor, heightScalingFactor);

        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_WIDTH, &origLabelWidth));
        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_HEIGHT, &origLabelHeight));

        newLabelWidth = (int)(widthScalingFactor * (float) origLabelWidth);
        newLabelHeight = (int)(heightScalingFactor * (float) origLabelHeight);

        /* find out if it is size to text */

        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_SIZE_TO_TEXT,
                                                       &sizeToText));

        if (!sizeToText)
        {
            errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_WIDTH, newLabelWidth));
            errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_HEIGHT, newLabelHeight));
        }

        errChk (GetCtrlAttribute (dupPanel, textCtrl, ATTR_WIDTH, &textMsgDimension));
        textMsgDimension -= 2;
        newDimensionToSetFontSize = (int)((float)textMsgDimension * whichFactor);
        errChk (newPointSize = GetNewPointSizeAfterChangingDimensions (dupPanel, textCtrl,
                                                                       newDimensionToSetFontSize,
                                                                       whichFactor, ATTR_WIDTH));

        errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_POINT_SIZE, newPointSize));
    }

Error:
    return error;

}

/***************************************************************************/

static int HandleXAndYAxisNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl, char *xAxisName, char *yAxisName,
                                 float whichFactor, float widthScalingFactor, float heightScalingFactor,
                                 int *newDimensionToSetFontSize)

/* This procedure is invoked if both the x and y axis have names.  The new point size is computed */
{
    int     xyNamesPointSize, xyNamesWidth;
    int     xNameLen, yNameLen, maxLen;
    char    fontName [MAXLEN] = {'\0'};
    int     error = kErrorNo;

    xNameLen = strlen (xAxisName);
    yNameLen = strlen (yAxisName);

    maxLen = Max (xNameLen, yNameLen);

    errChk (SetCtrlVal (dupPanel, textCtrl, (maxLen == xNameLen) ? xAxisName : yAxisName));
    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_FONT, fontName));
    errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_FONT, fontName));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XYNAME_POINT_SIZE, &xyNamesPointSize));
    errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_POINT_SIZE, xyNamesPointSize));
    errChk (GetCtrlAttribute (dupPanel, textCtrl, ATTR_WIDTH, &xyNamesWidth));
    xyNamesWidth -= 2;
    *newDimensionToSetFontSize = (int)((float)xyNamesWidth * whichFactor);

Error:
    return error;
}

/***************************************************************************/

static int HandleXOrYAxisNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl, int textCtrl, char *axisName, float whichFactor,
                                float widthScalingFactor, float heightScalingFactor,
                                int *newDimensionToSetFontSize)

/* This module is invoked if either the x axis or the y axis has names.  The new point size is computed */

{
    int     xyNamesPointSize, xyNamesWidth;
    char    fontName [MAXLEN] = {'\0'};
    int     error = kErrorNo;

    errChk (SetCtrlVal (dupPanel, textCtrl, axisName));

    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_FONT, fontName));
    errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_FONT, fontName));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XYNAME_POINT_SIZE, &xyNamesPointSize));
    errChk (SetCtrlAttribute (dupPanel, textCtrl, ATTR_TEXT_POINT_SIZE, xyNamesPointSize));
    errChk (GetCtrlAttribute (dupPanel, textCtrl, ATTR_WIDTH, &xyNamesWidth));
    xyNamesWidth -= 2;
    *newDimensionToSetFontSize = (int)((float)xyNamesWidth * whichFactor);

Error:
    return error;
}

/***************************************************************************/

static int UpdateAxisLabelsAndNames (int origPanel, int origCtrl, int dupPanel, int dupCtrl,
                                                                  int   textCtrl,
                                                                  float widthScalingFactor,
                                                                  float heightScalingFactor)

/* Updates the point size of axis labels and names */

{
    int     newPointSize, xyLabelPointSize, xyNamePointSize, origPointSize, titlePointSize;
    float   whichFactor, whichDimension;
    int     newDimensionToSetFontSize;
    char    xAxisName [MAXLEN] = {'\0'}, yAxisName [MAXLEN] = {'\0'};
    char    xyFontName [MAXLEN] = {'\0'}, titleFontName [MAXLEN] = {'\0'}, labelFontName [MAXLEN] = {'\0'};
    int     error = kErrorNo;

    /* Update the axis names first */

    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_FONT, titleFontName));

    /* If the labels and the names have same fonts, set the fontsize to same */

    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_FONT, xyFontName));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_POINT_SIZE, &titlePointSize));
    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_POINT_SIZE, &xyNamePointSize));

    /* First check to see if the axis names and control title had the same font and point size.
       If so, why bother to do extra work?  Simply set the point size of the axis names to be
       the same as the point size of the control title. */

    if ((strcmp (titleFontName, xyFontName) == 0) && (titlePointSize == xyNamePointSize))
    {
        errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_POINT_SIZE, &titlePointSize));
        errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_POINT_SIZE, titlePointSize));
    }
    else
    {
        /* Pick the longer of the two axis names as the one to determine the dimension which will
           help in getting the new point size */

        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XNAME, xAxisName));
        errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_YNAME, yAxisName));

        whichFactor = Min (widthScalingFactor, heightScalingFactor);

        whichDimension = (float)((whichFactor == widthScalingFactor) ? ATTR_WIDTH : ATTR_HEIGHT);

        if (strlen (xAxisName) && strlen (yAxisName))
            HandleXAndYAxisNames (origPanel, origCtrl, dupPanel, dupCtrl, textCtrl, xAxisName, yAxisName,
                                  whichFactor, widthScalingFactor, heightScalingFactor, &newDimensionToSetFontSize);
        else
            if (strlen (xAxisName))
            {
                errChk (HandleXOrYAxisNames (origPanel, origCtrl, dupPanel, dupCtrl, textCtrl, xAxisName, whichFactor, widthScalingFactor,
                                             heightScalingFactor, &newDimensionToSetFontSize));
            }
            else
                if (strlen (yAxisName))
                {
                    errChk (HandleXOrYAxisNames (origPanel, origCtrl, dupPanel, dupCtrl, textCtrl, yAxisName, whichFactor, widthScalingFactor,
                                         heightScalingFactor, &newDimensionToSetFontSize));
                }

        if (strlen (xAxisName) || strlen (yAxisName))
        {
            errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XYNAME_POINT_SIZE, &origPointSize));
            errChk (newPointSize = GetNewPointSizeAfterChangingDimensions (dupPanel, textCtrl,
                                                                           newDimensionToSetFontSize,
                                                                           whichFactor, ATTR_WIDTH));
            errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_POINT_SIZE, newPointSize));
        }
    }

    /* Now deal with labels */

    /* Do the same thing for xy labels.  First check to see if labels and control title have the
       same font and point size.  If so, set their point sizes to be the same.  If not, then perform
       a similar check between xy labels and xy names, else simply scale up the point size by the
       minimum of the scaling factors */

    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYLABEL_FONT, labelFontName));
    errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_FONT, xyFontName));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_POINT_SIZE, &titlePointSize));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XYNAME_POINT_SIZE, &xyNamePointSize));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_XYLABEL_POINT_SIZE, &xyLabelPointSize));

    if ((strcmp (titleFontName, labelFontName) == 0) && (titlePointSize == xyLabelPointSize))
    {
        errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_LABEL_POINT_SIZE, &titlePointSize));
        errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYLABEL_POINT_SIZE, titlePointSize));
    }
    else
    if ((strcmp (xyFontName, labelFontName) == 0) && (xyNamePointSize == xyLabelPointSize))
    {
        errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYNAME_POINT_SIZE, &xyNamePointSize));
        errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYLABEL_POINT_SIZE, xyNamePointSize));
    }
    else
    {
        whichFactor = Min (widthScalingFactor, heightScalingFactor);
        errChk (GetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYLABEL_POINT_SIZE, &xyLabelPointSize));
        xyLabelPointSize = (int)floor ((float) xyLabelPointSize * whichFactor + .5);
        errChk (SetCtrlAttribute (dupPanel, dupCtrl, ATTR_XYLABEL_POINT_SIZE, xyLabelPointSize));
    }

Error:
    return error;
}

/***************************************************************************/

static int UpdateGraphOrStripChartLabelsToNewPointSizes (int origPanel, int origCtrl, int dupPanel, int dupCtrl,
                                                         float widthScalingFactor,
                                                         float heightScalingFactor)

{
    char    fontName [MAXLEN] = {'\0'};
    int     textCtrl=0;
    int     error = kErrorNo;

    /* Create a text control */

    errChk (textCtrl = NewCtrl (dupPanel, CTRL_TEXT_MSG, "Text Message", 50, 50));

    errChk (UpdateTitleLabel (origPanel, origCtrl, dupPanel, dupCtrl, textCtrl, widthScalingFactor,
                                                              heightScalingFactor));
    UpdateAxisLabelsAndNames (origPanel, origCtrl, dupPanel, dupCtrl, textCtrl, widthScalingFactor,
                                                              heightScalingFactor);

Error:
    if (textCtrl > 0)
        DiscardCtrl (dupPanel, textCtrl);

    return error;
}

/***************************************************************************/

static int PlaceLabel (int panel, int controlId, int attrCtrlDim, int attrCtrl, int attrLabel, float percentage)

{
    int     ctrlDimSize, ctrlOffset;
    float   howMuchFromStartOfControl;
    int     error = kErrorNo;

    if (percentage)
    {
        errChk (GetCtrlAttribute (panel, controlId, attrCtrlDim, &ctrlDimSize));
        howMuchFromStartOfControl = (percentage * (float)ctrlDimSize)/(float)100;
        errChk (GetCtrlAttribute (panel, controlId, attrCtrl, &ctrlOffset));
        errChk (SetCtrlAttribute (panel, controlId, attrLabel, ctrlOffset + (int) howMuchFromStartOfControl));
    }

Error:
    return error;
}

/***************************************************************************/

static int PlaceControlLabelAtTheRightPlace (int origPanel, int origCtrl, int dupPanel, int dupCtrl)

/* Place the label of the control in proportion to its change in size */

{
    float   horzPositionPercentageDiff, vertPositionPercentageDiff;
    int     error = kErrorNo;

    errChk (ComputeLabelPlacementPercentage (origPanel,
                                             origCtrl, ATTR_WIDTH, ATTR_LEFT,
                                             ATTR_LABEL_LEFT, &horzPositionPercentageDiff));
    errChk (ComputeLabelPlacementPercentage (origPanel,
                                             origCtrl, ATTR_HEIGHT, ATTR_TOP,
                                             ATTR_LABEL_TOP, &vertPositionPercentageDiff));

    errChk (PlaceLabel (dupPanel, dupCtrl, ATTR_WIDTH, ATTR_LEFT, ATTR_LABEL_LEFT, horzPositionPercentageDiff));
    errChk (PlaceLabel (dupPanel, dupCtrl, ATTR_HEIGHT, ATTR_TOP, ATTR_LABEL_TOP, vertPositionPercentageDiff));

Error:
    return error;
}

/***************************************************************************/

static int ConvertMilliMetersToDots (int dataInMilliMeters, Dimension dim)

{
    float   dimInInches;
    int     resolution, dots;
    int     error = kErrorNo;

    dimInInches = (float)dataInMilliMeters/(float)CONV_CONST;

    if (dim == kWidth)
    {
        errChk (GetValidatedPrintAttribute (ATTR_XRESOLUTION, &resolution));
    }
    else
    {
        errChk (GetValidatedPrintAttribute (ATTR_YRESOLUTION, &resolution));
    }

    dots = (int)(dimInInches*(float)resolution);

Error:
    if (error < 0)
        return error;
    else
        return dots;
}

/***************************************************************************/

static int ComputeScalingFactorsFromPrintAttributes  (int origBoundingRectWidth, int origBoundingRectHeight,
                                                      float *widthScalingFactor, float *heightScalingFactor)

{
    int     paperWidth, paperHeight;
    int     widthInDots=0, heightInDots=0;
    int     orientation;
    int     error = kErrorNo;

    errChk(GetValidatedPrintAttribute (ATTR_PAPER_WIDTH, &paperWidth));
    errChk (GetValidatedPrintAttribute (ATTR_PAPER_HEIGHT, &paperHeight));
    errChk (GetPrintAttribute (ATTR_ORIENTATION, &orientation));

    if (paperWidth)
        errChk (widthInDots = ConvertMilliMetersToDots (paperWidth, kWidth));
    if (paperHeight)
        errChk (heightInDots = ConvertMilliMetersToDots (paperHeight, kHeight));

    if ((paperWidth == 0) && (paperHeight == 0))
        *widthScalingFactor = *heightScalingFactor = (float)1;
    else
    if (paperHeight == 0)
    {
        if (orientation == VAL_PORTRAIT)
        {

            *widthScalingFactor = (float)widthInDots/(float)origBoundingRectWidth;
            if (!(*widthScalingFactor))
                *widthScalingFactor = (float)1;
            *heightScalingFactor = *widthScalingFactor;
        }
        else
        {
            *heightScalingFactor = (float)widthInDots/(float)origBoundingRectHeight;
            if (!(*heightScalingFactor))
                *heightScalingFactor = (float)1;
            *widthScalingFactor = *heightScalingFactor;
        }
    }
    else
    if (paperWidth == 0)
    {
        if (orientation == VAL_PORTRAIT)
        {
            *heightScalingFactor = (float)heightInDots/(float)origBoundingRectHeight;
            if (!(*heightScalingFactor))
                *heightScalingFactor = (float)1;
            *widthScalingFactor = *heightScalingFactor;
        }
        else
        {
            *widthScalingFactor = (float)heightInDots/(float)origBoundingRectWidth;
            if (!(*widthScalingFactor))
                *widthScalingFactor = (float)1;
            *heightScalingFactor = *widthScalingFactor;
        }
    }
    else
    {
        if (orientation == VAL_PORTRAIT)
        {
            *widthScalingFactor = (float) widthInDots/(float) origBoundingRectWidth;
            *heightScalingFactor = (float) heightInDots/(float) origBoundingRectHeight;
        }
        else
        {
            *widthScalingFactor = (float) heightInDots/origBoundingRectWidth;
            *heightScalingFactor = (float) widthInDots/origBoundingRectHeight;
        }
    }

Error:
    return error;
}

/***************************************************************************/

static int DuplicateAndSetTheSizeOfTheControl (int origPanel, int origCtrl,
                                         int *dupPanel, int *dupCtrl,
                                         float widthScalingFactor, float heightScalingFactor)

/* This function duplicates the control and resizes it according to
   the print attributes */

{
    int     newCtrlWidth, newCtrlHeight, origCtrlWidth, origCtrlHeight;
    char    duplicateTitle [MAXLEN] = {'\0'};
    int     error = kErrorNo;

    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_HEIGHT, &origCtrlHeight));
    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_WIDTH, &origCtrlWidth));

    errChk (GetCtrlAttribute (origPanel, origCtrl, ATTR_LABEL_TEXT, duplicateTitle));

    errChk (*dupPanel = NewPanel (0, kDupPanelTitle, kDupPanelTop, kDupPanelLeft,
                                             kDupPanelHeight, kDupPanelWidth));
    errChk (SetPanelAttribute (*dupPanel, ATTR_SCROLL_BARS, VAL_BOTH_SCROLL_BARS));

    errChk (*dupCtrl = DuplicateCtrl (origPanel, origCtrl, *dupPanel, duplicateTitle,
                                                   kDupCtrlTop, kDupCtrlLeft));

    newCtrlWidth = (int)(widthScalingFactor * (float) origCtrlWidth);
    newCtrlHeight = (int)(heightScalingFactor * (float) origCtrlHeight);

    errChk (SetCtrlAttribute (*dupPanel, *dupCtrl, ATTR_WIDTH, newCtrlWidth));
    errChk (SetCtrlAttribute (*dupPanel, *dupCtrl, ATTR_HEIGHT, newCtrlHeight));

Error:
    if (error < 0)
        if (*dupPanel > 0)
        {
            DiscardPanel (*dupPanel);
            *dupPanel = 0;
        }
    return error;
}

/***************************************************************************/

static int GetScalingFactorsFromBoundingRect (int origPanel, int origCtrl, float *widthScalingFactor,
                                              float *heightScalingFactor)

/* Get the scaling factors */

{
    int     origRectTop, origRectLeft, origRectHeight, origRectWidth;
    int     newRectWidth, newRectHeight;
    int     error = kErrorNo;

    errChk (GetCtrlBoundingRect (origPanel, origCtrl, &origRectTop, &origRectLeft,
                         &origRectHeight, &origRectWidth));

    errChk (ComputeScalingFactorsFromPrintAttributes (origRectWidth, origRectHeight,
                                              widthScalingFactor, heightScalingFactor));

    newRectHeight = (int)((float)origRectHeight * *heightScalingFactor);
    newRectWidth = (int)((float)origRectWidth * *widthScalingFactor);

Error:
    return error;
}

/***************************************************************************/

static void MakeNumTableEntriesEven (char **table)

{
    int i = 0;

    while (table [i] != NULL)
        i++;

    /* If number of entries in the table are non-even, truncate the table by
       removing the last item */

    if ((i%2)!=0)
        table [i-1] = NULL;
}

/***************************************************************************/

static int CaseInSensitiveStringCompare (char *string1, char *string2)

/* Compares two strings non case sensitively */

{
    if ((!string1) && (!string2))
        return 0;
    if (!string1)
        return -1;
    if (!string2)
        return 1;

    while (TRUE)
    {
        if (!(*string1) && !(*string2))
            return 0;
        if (!(*string1))
            return -1;
        else
        if (!(*string2))
            return 1;
        else
        if (toupper (*string1) > toupper (*string2))
            return 1;
        if (toupper (*string1) < toupper (*string2))
            return -1;
        else
        {
            string1++;
            string2++;
        }
    }
}

/***************************************************************************/

static int SearchAndSubstituteFont (int panel, int controlId, char *fontToSearch, char *fontToSubstitute)

/* Search if the control title, axis names or axis labels are of a certain font.  If so,
   replace it with the new font */
{
    int     i;
    char    currentFont [MAXLEN] = {'\0'};
    int     error = kErrorNo;

    for (i=0; fontNameAttributes [i] != kEndOfList; i++)
    {
        errChk (GetCtrlAttribute (panel, controlId, fontNameAttributes [i], currentFont));

        if ((CaseInSensitiveStringCompare (currentFont, fontToSearch)) == 0)
            errChk (SetCtrlAttribute (panel, controlId, fontNameAttributes [i], fontToSubstitute));
    }

Error:
    return error;
}

/***************************************************************************/

static int SubstituteFonts (int panel, int controlId, char **fontSubstitutionTable)

/* This module substitutes fonts, from a font substitution table. */

{
    int i, error = kErrorNo;

    if (!fontSubstitutionTable)
        goto Error;
        
    for (i = 0; fontSubstitutionTable[i] && fontSubstitutionTable[i+1]; i += 2)
        errChk(SearchAndSubstituteFont (panel, controlId, fontSubstitutionTable[i],
            fontSubstitutionTable [i+1]));

Error:
    return error;
}

/***************************************************************************/

int CVIFUNC PrintGraphHiRes (int panel, int controlId, char *fileName,
    char **fontSubstitutionTable)

/* The function takes the following sequence of events :-

   (1)  Computes scaling factors
   (2)  Duplicates and resizes the control to print
   (3)  Substitutes fonts from a font substitution table
   (4)  Handles a special case of text plots
   (5)  Updates fonts to new point sizes
   (6)  Places the control label or title of the control at the proper place after
        the control has been resized
   (7)  Finally, prints the duplicate control */

{
    int     dupPanel = 0, dupCtrl, ctrlStyle;
    float   heightScalingFactor, widthScalingFactor;
    int     error = kErrorNo;


    errChk (GetCtrlAttribute (panel, controlId, ATTR_CTRL_STYLE, &ctrlStyle));
    if (ctrlStyle != CTRL_GRAPH && ctrlStyle != CTRL_STRIP_CHART)
        errChk(-10);

    errChk (GetScalingFactorsFromBoundingRect (panel, controlId, &widthScalingFactor, &heightScalingFactor));
    errChk (DuplicateAndSetTheSizeOfTheControl (panel, controlId, &dupPanel, &dupCtrl,
                                        widthScalingFactor, heightScalingFactor));

    if (ctrlStyle == CTRL_GRAPH)
        SetCtrlAttribute (dupPanel, dupCtrl, ATTR_SMOOTH_UPDATE, 0);    /* to save memory */

    errChk (SubstituteFonts (dupPanel, dupCtrl, fontSubstitutionTable));

    errChk (HandleTextPlots (dupPanel, dupCtrl, widthScalingFactor, heightScalingFactor));
    errChk (UpdateGraphOrStripChartLabelsToNewPointSizes (panel, controlId, dupPanel, dupCtrl,
                                                  widthScalingFactor, heightScalingFactor));

    errChk (PlaceControlLabelAtTheRightPlace (panel, controlId, dupPanel, dupCtrl));
    error = PrintCtrl (dupPanel, dupCtrl, fileName, 1, 0);

Error:
    if (dupPanel > 0)
    {
        DiscardPanel (dupPanel);
        dupPanel = 0;
    }
    return error;
}

/***************************************************************************/
